part of 'training_plan_bloc.dart';

abstract class TrainingPlanState extends Equatable {
  const TrainingPlanState();

  @override
  List<Object> get props => [];
}

class TrainingPlanInitial extends TrainingPlanState {
  const TrainingPlanInitial();
}

class TrainingPlanLoading extends TrainingPlanState {
  const TrainingPlanLoading();
}

class TrainingPlanReady extends TrainingPlanState {
  const TrainingPlanReady();
}

class TrainingPlanFinished extends TrainingPlanState {
  const TrainingPlanFinished();
}

class TrainingPlanError extends TrainingPlanState {
  final String message;
  const TrainingPlanError({required this.message});

  @override
  List<Object> get props => [message];
}
